#! /usr/bin/env node
'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _regexgen = require('regexgen');

var _packageData = require('../packageData');

var _packageData2 = _interopRequireDefault(_packageData);

var _writeFile = require('./helpers/writeFile');

var _writeFile2 = _interopRequireDefault(_writeFile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function generateRegex(rawData, display) {
  var unicode = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  if (unicode) {
    console.log('Generating unicode aware ' + display + ' regex pattern');
  } else {
    console.log('Generating ' + display + ' regex pattern');
  }

  var flags = unicode ? 'u' : '';
  var fileName = display === 'both' ? 'index' : display;
  var codePointGroups = {
    4: new _regexgen.Trie(),
    3: new _regexgen.Trie(),
    2: new _regexgen.Trie(),
    1: new _regexgen.Trie()
  };

  return _promise2.default.resolve(rawData).then(function (data) {
    data.forEach(function (emoji) {
      var group = emoji.codepoint.length;

      if (group <= 0 || group > 4) {
        return;
      }

      if (emoji.emoji && (display === 'emoji' || display === 'both')) {
        codePointGroups[group].add(emoji.emoji);
      }

      if (emoji.text && (display === 'text' || display === 'both')) {
        codePointGroups[group].add(emoji.text);
      }
    });

    return codePointGroups;
  }).then(function (groups) {
    if (display === 'text') {
      return groups[1].toRegExp(flags).source;
    }

    return [4, 3, 2, 1].map(function (group) {
      return '(?:' + groups[group].toRegExp(flags).source + ')';
    }).join('|');
  }).then(function (regex) {
    return (0, _writeFile2.default)(_path2.default.join(__dirname, '../../regex/' + (unicode ? 'es/' : '') + fileName + '.js'), regex, function (pattern) {
      return 'module.exports = /' + pattern + '/;\n';
    }, false);
  });
}
/**
 * @copyright   2017, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */


function generateExtra(rawData) {
  console.log('Generating shortnames regex pattern');

  return _promise2.default.resolve(rawData).then(function (data) {
    return (0, _writeFile2.default)(_path2.default.join(__dirname, '../../regex/shortname.js'), ':[-+\\w]+:', function (pattern) {
      return 'module.exports = /' + pattern + '/;\n';
    }, false);
  });
}

var data = (0, _packageData2.default)();

_promise2.default.all([generateRegex(data, 'both'), generateRegex(data, 'both', true), generateRegex(data, 'emoji'), generateRegex(data, 'emoji', true), generateRegex(data, 'text'), generateRegex(data, 'text', true), generateExtra(data)]).then(function () {
  console.log(_chalk2.default.green('Regex generated successfully'));
}).catch(function (error) {
  console.log(_chalk2.default.red('Failed to generate regex pattern'));
  console.log(_chalk2.default.gray(error.message));
});